/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#ifndef RECOVERY_DATA_HPP
#define RECOVERY_DATA_HPP

#include <set>
#include <map>
#include <vector>
#include <DxcDataTypes.hpp>
#include <DxcDefs.hpp>

namespace Dxc
{
/**
 * @brief Defines the data structure for system Recovery Info.
 *
 * Recovery information consists of a hypothetical set of fault modes (with
 * estimated parameters), a set of commands that represent a recovery action
 * for the fault modes, and a cost for performing these actions.
 */
class DXCAPI_EXPORT RecoveryData : public DxcData
{
public:
    /**
     * @brief Constructor to initialize the RecoveryData with timestamp.
     *
     * @param timestamp - message timestamp.
     * @param faults - hypothetical fault set
     * @param commands - recovery actions
     * @param cost - cost of taking the actions
     * @param notes - additional information, such as errors.
     */
    RecoveryData(Timestamp timestamp,
                 const FaultSet& fs = FaultSet(),
                 const CommandSet& commands = CommandSet(),
                 double cost = -1,
                 const std::string& notes = "");

    /**
     * @brief Constructor to initialize the RecoveryData with timestamp.
     *
     * @param faults - hypothetical fault set
     * @param commands - recovery actions
     * @param cost - cost of taking the actions
     * @param notes - additional information, such as errors.
     */
    RecoveryData(const FaultSet& fs = FaultSet(),
                 const CommandSet& commands = CommandSet(),
                 double cost = -1,
                 const std::string& notes = "");

    const FaultSet* getFaults() const;
    const CommandSet* getCommands() const;
    double getCost() const;
    std::string getNotes() const;

    virtual RecoveryData* clone() const;

    virtual std::ostream &put(std::ostream &) const;

private:
    FaultSet faults_;
    CommandSet commands_;
    double cost_;
    std::string notes_;
};
}

#endif

