package org.dxc.api.datatypes;

import java.io.OutputStream;
import java.io.PrintStream;

/**
 * Defines the data structure for system Recovery Info.
 * 
 * Recovery information consists of a hypothetical set of fault modes (with
 * estimated parameters), a set of commands that represent a recovery action for
 * the fault modes, and a cost for performing these actions.
 */
public class RecoveryData extends DxcData {
    private FaultSet faults;
    private CommandSet commands;
    private double cost;
    private String notes;

    /**
     * Constructor to initialize the RecoveryData with timestamp.
     * 
     * @param timestamp
     *            - message timestamp.
     * @param faults
     *            - hypothetical fault set
     * @param commands
     *            - recovery actions
     * @param cost
     *            - cost of taking the actions
     * @param notes
     *            - additional information, such as errors.
     */
    public RecoveryData(long timestamp, FaultSet faults, CommandSet commands,
            double cost, String notes) {
        super(timestamp);
        this.faults = faults;
        this.commands = commands;
        this.cost = cost;
        this.notes = notes;
    }

    public RecoveryData(FaultSet faults, CommandSet commands, double cost,
            String notes) {
        super();
        this.faults = faults;
        this.commands = commands;
        this.cost = cost;
        this.notes = notes;
    }

    /**
     * Constructor to initialize the RecoveryData with nothing but fault set.
     * 
     * @param faults
     *            - hypothetical fault set
     */
    public RecoveryData(FaultSet faults) {
        super();
        this.faults = faults;
        this.commands = new CommandSet();
        this.cost = -1;
        this.notes = "";
    }

    public RecoveryData(FaultSet faults, CommandSet commands) {
        super();
        this.faults = faults;
        this.commands = commands;
        this.cost = -1;
        this.notes = "";
    }

    public FaultSet getFaults() {
        return faults;
    }

    public CommandSet getCommands() {
        return commands;
    }

    public double getCost() {
        return cost;
    }

    public String getNotes() {
        return notes;
    }

    @Override
    public OutputStream put(OutputStream os) {
        PrintStream out = new PrintStream(os);
        out.append("recoveryInfo @");
        out.append(Long.toString(this.getTimeStamp()));
        if (this.getSource() != null)
            out.append(" <" + this.getSource() + ">");

        out.append(" faults = " + faults + ",");
        out.append(" commands = ");

        out.append(commands.toString());

        out.append(",");
        out.append(" cost = " + cost + ",");
        out.append(" notes = '" + notes + "';\n");

        return out;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = super.hashCode();
        result = prime * result
                + ((commands == null) ? 0 : commands.hashCode());
        long temp;
        temp = Double.doubleToLongBits(cost);
        result = prime * result + (int) (temp ^ (temp >>> 32));
        result = prime * result + ((faults == null) ? 0 : faults.hashCode());
        result = prime * result + ((notes == null) ? 0 : notes.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (!super.equals(obj))
            return false;
        if (getClass() != obj.getClass())
            return false;
        RecoveryData other = (RecoveryData) obj;
        if (commands == null) {
            if (other.commands != null)
                return false;
        } else if (!commands.equals(other.commands))
            return false;
        if (Double.doubleToLongBits(cost) != Double
                .doubleToLongBits(other.cost))
            return false;
        if (faults == null) {
            if (other.faults != null)
                return false;
        } else if (!faults.equals(other.faults))
            return false;
        if (notes == null) {
            if (other.notes != null)
                return false;
        } else if (!notes.equals(other.notes))
            return false;
        return true;
    }

}
