/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.evaluator;

import org.dxc.api.DxcConfig;
import org.dxc.api.datatypes.ScenarioData;

import java.io.File;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

public class Evaluator {
    private static Map<String, DAResult> daResults;
    private static Map<String, List<DAResult>> resultsBySystem;
    private static boolean printStats;

    public static void main(String[] args) {

        daResults = new TreeMap<String, DAResult>();
        resultsBySystem = new TreeMap<String, List<DAResult>>();

        try {
            EvalConfig ec = EvalConfig.getInstance();
            printStats = ec.printStats();
        } catch (Exception ex) {
            System.out.println("Unable to open evaluator config: "
                    + ex.toString());
            System.exit(1);
        }

        String dir = "";

        try {
            dir = DxcConfig.getInstance().get("RESULTS_DIR");
        } catch (Exception ex) {
            System.out.println("Unable to find results dir: " + ex.toString());
            System.exit(1);
        }

        File resultsDir = new File(dir);
        Set<String> contents = new TreeSet<String>();

        contents.addAll(Arrays.asList(resultsDir.list()));
        Iterator<String> i = contents.iterator();

        String s = i.hasNext() ? i.next() : "";

        while (!s.equals("")) {
            if (s.endsWith(".scn")) {
                String daSystemPair = getDASystemPair(s);
                DAResult dar = null;

                try {
                    dar = getDAResults(daSystemPair);
                } catch (Exception ex) {
                    System.out.append(ex.toString());
                    System.exit(1);
                }

                while (s.contains(daSystemPair + " - ")) {
                    String system = daSystemPair.substring(daSystemPair
                            .lastIndexOf(" - ") + 3);
                    String daName = daSystemPair.substring(0, daSystemPair
                            .lastIndexOf(" - "));

                    // temporary fix to populate missing scenario info in
                    // earlier runs
                    ScenarioData sd = new ScenarioData(s, system, "", daName,
                            "", "", 0, "");

                    try {
                        ScenarioResult sr = new ScenarioResultFile(dir + "/"
                                + s, sd);

                        if (!sr.hasErrors()) {
                            dar.addScenario(sr);

                            if (printStats) {
                                sr.print();
                            }
                        } else {
                            System.out.println("Scenario " + s
                                    + " contains errors. Discarding.");
                            System.out.println();
                        }
                    } catch (Exception ex) {
                        System.out.println(ex);
                        System.out.println("Loading scenario " + s
                                + " failed. Discarding.");
                        System.out.println();
                    }

                    s = i.hasNext() ? i.next() : "";
                }

                dar.calculate();
            } else {
                s = i.hasNext() ? i.next() : "";
            }
        }

        Iterator<DAResult> j = daResults.values().iterator();
        if (!j.hasNext())
            System.out.println("No results. Exiting.");

        while (j.hasNext()) {
            DAResult d = j.next();
            d.print();
            System.out.println();

            if (resultsBySystem.containsKey(d.getSystem())) {
                List<DAResult> dal = resultsBySystem.get(d.getSystem());
                dal.add(d);
            } else {
                List<DAResult> dal = new LinkedList<DAResult>();
                dal.add(d);
                resultsBySystem.put(d.getSystem(), dal);
            }
        }

        try {
            ScoreKeeper sk = new ScoreKeeper();

            for (String sys : resultsBySystem.keySet()) {
                System.out.println("Scores for system " + sys + ": ");
                sk.scoreDAs(resultsBySystem.get(sys));
                System.out.println();
            }

        } catch (Exception ex) {
            System.out.println(ex);
        } finally {
            org.dxc.evaluator.metrics.OracleInterface.shutdown();
        }

        System.exit(0);
    }

    private static String getDASystemPair(String filename) {
        String s = null;
        try {
            s = filename.substring(
                    filename.lastIndexOf(File.separatorChar) + 1, filename
                            .lastIndexOf(" - "));
        } catch (Exception ex) {
            System.out
                    .println("Error: scenario result filename must have the format: ");
            System.out.println("[DA Name] - [System Name] - [Scenario].scn");
            System.out.println("Scenario: " + filename);
        }
        return s;
    }

    private static DAResult getDAResults(String daSystemPair) throws Exception {
        DAResult dar;

        String system = daSystemPair
                .substring(daSystemPair.lastIndexOf(" - ") + 3);
        if (daResults.containsKey(daSystemPair)) {
            dar = daResults.get(daSystemPair);
        } else {
            String daName = daSystemPair.substring(0, daSystemPair
                    .lastIndexOf(" - "));
            dar = new DAResult(daName, system);
            daResults.put(daSystemPair, dar);
        }

        return dar;
    }
}
